@@ -18,6 +18,7 @@
  */
 
 #define _GNU_SOURCE
+#define _LGPL_SOURCE
 #include <assert.h>
 #include <poll.h>
 #include <pthread.h>
@@ -285,6 +286,17 @@ static void free_channel_rcu(struct rcu_head *head)
 	struct lttng_consumer_channel *channel =
 		caa_container_of(node, struct lttng_consumer_channel, node);
 
+	switch (consumer_data.type) {
+	case LTTNG_CONSUMER_KERNEL:
+		break;
+	case LTTNG_CONSUMER32_UST:
+	case LTTNG_CONSUMER64_UST:
+		lttng_ustconsumer_free_channel(channel);
+		break;
+	default:
+		ERR("Unknown consumer_data type");
+		abort();
+	}
 	free(channel);
 }
 
@@ -562,6 +574,7 @@ struct lttng_consumer_stream *consumer_allocate_stream(uint64_t channel_key,
 	stream->endpoint_status = CONSUMER_ENDPOINT_ACTIVE;
 	stream->index_fd = -1;
 	pthread_mutex_init(&stream->lock, NULL);
+	pthread_mutex_init(&stream->metadata_timer_lock, NULL);
 
 	/* If channel is the metadata, flag this stream as metadata. */
 	if (type == CONSUMER_CHANNEL_TYPE_METADATA) {
@@ -935,7 +948,9 @@ struct lttng_consumer_channel *consumer_allocate_channel(uint64_t key,
 		uint64_t tracefile_count,
 		uint64_t session_id_per_pid,
 		unsigned int monitor,
-		unsigned int live_timer_interval)
+		unsigned int live_timer_interval,
+		const char *root_shm_path,
+		const char *shm_path)
 {
 	struct lttng_consumer_channel *channel;
 
@@ -992,6 +1007,15 @@ struct lttng_consumer_channel *consumer_allocate_channel(uint64_t key,
 	strncpy(channel->name, name, sizeof(channel->name));
 	channel->name[sizeof(channel->name) - 1] = '\0';
 
+	if (root_shm_path) {
+		strncpy(channel->root_shm_path, root_shm_path, sizeof(channel->root_shm_path));
+		channel->root_shm_path[sizeof(channel->root_shm_path) - 1] = '\0';
+	}
+	if (shm_path) {
+		strncpy(channel->shm_path, shm_path, sizeof(channel->shm_path));
+		channel->shm_path[sizeof(channel->shm_path) - 1] = '\0';
+	}
+
 	lttng_ht_node_init_u64(&channel->node, channel->key);
 
 	channel->wait_fd = -1;
@@ -1430,7 +1454,7 @@ static int write_relayd_metadata_id(int fd,
 	ret = lttng_write(fd, (void *) &hdr, sizeof(hdr));
 	if (ret < sizeof(hdr)) {
 		/*
-		 * This error means that the fd's end is closed so ignore the perror
+		 * This error means that the fd's end is closed so ignore the PERROR
 		 * not to clubber the error output since this can happen in a normal
 		 * code path.
 		 */
@@ -2216,26 +2240,22 @@ restart:
 			}
 
 			if (pollfd == lttng_pipe_get_readfd(ctx->consumer_metadata_pipe)) {
-				if (revents & (LPOLLERR | LPOLLHUP )) {
-					DBG("Metadata thread pipe hung up");
-					/*
-					 * Remove the pipe from the poll set and continue the loop
-					 * since their might be data to consume.
-					 */
-					lttng_poll_del(&events,
-							lttng_pipe_get_readfd(ctx->consumer_metadata_pipe));
-					lttng_pipe_read_close(ctx->consumer_metadata_pipe);
-					continue;
-				} else if (revents & LPOLLIN) {
+				if (revents & LPOLLIN) {
 					ssize_t pipe_len;
 
 					pipe_len = lttng_pipe_read(ctx->consumer_metadata_pipe,
 							&stream, sizeof(stream));
 					if (pipe_len < sizeof(stream)) {
-						PERROR("read metadata stream");
+						if (pipe_len < 0) {
+							PERROR("read metadata stream");
+						}
 						/*
-						 * Continue here to handle the rest of the streams.
+						 * Remove the pipe from the poll set and continue the loop
+						 * since their might be data to consume.
 						 */
+						lttng_poll_del(&events,
+								lttng_pipe_get_readfd(ctx->consumer_metadata_pipe));
+						lttng_pipe_read_close(ctx->consumer_metadata_pipe);
 						continue;
 					}
 
@@ -2252,6 +2272,19 @@ restart:
 					/* Add metadata stream to the global poll events list */
 					lttng_poll_add(&events, stream->wait_fd,
 							LPOLLIN | LPOLLPRI | LPOLLHUP);
+				} else if (revents & (LPOLLERR | LPOLLHUP)) {
+					DBG("Metadata thread pipe hung up");
+					/*
+					 * Remove the pipe from the poll set and continue the loop
+					 * since their might be data to consume.
+					 */
+					lttng_poll_del(&events,
+							lttng_pipe_get_readfd(ctx->consumer_metadata_pipe));
+					lttng_pipe_read_close(ctx->consumer_metadata_pipe);
+					continue;
+				} else {
+					ERR("Unexpected poll events %u for sock %d", revents, pollfd);
+					goto end;
 				}
 
 				/* Handle other stream */
@@ -2270,8 +2303,30 @@ restart:
 			stream = caa_container_of(node, struct lttng_consumer_stream,
 					node);
 
-			/* Check for error event */
-			if (revents & (LPOLLERR | LPOLLHUP)) {
+			if (revents & (LPOLLIN | LPOLLPRI)) {
+				/* Get the data out of the metadata file descriptor */
+				DBG("Metadata available on fd %d", pollfd);
+				assert(stream->wait_fd == pollfd);
+
+				do {
+					health_code_update();
+
+					len = ctx->on_buffer_ready(stream, ctx);
+					/*
+					 * We don't check the return value here since if we get
+					 * a negative len, it means an error occured thus we
+					 * simply remove it from the poll set and free the
+					 * stream.
+					 */
+				} while (len > 0);
+
+				/* It's ok to have an unavailable sub-buffer */
+				if (len < 0 && len != -EAGAIN && len != -ENODATA) {
+					/* Clean up stream from consumer and free it. */
+					lttng_poll_del(&events, stream->wait_fd);
+					consumer_del_metadata_stream(stream, metadata_ht);
+				}
+			} else if (revents & (LPOLLERR | LPOLLHUP)) {
 				DBG("Metadata fd %d is hup|err.", pollfd);
 				if (!stream->hangup_flush_done
 						&& (consumer_data.type == LTTNG_CONSUMER32_UST
@@ -2299,31 +2354,11 @@ restart:
 				 * and securely free the stream.
 				 */
 				consumer_del_metadata_stream(stream, metadata_ht);
-			} else if (revents & (LPOLLIN | LPOLLPRI)) {
-				/* Get the data out of the metadata file descriptor */
-				DBG("Metadata available on fd %d", pollfd);
-				assert(stream->wait_fd == pollfd);
-
-				do {
-					health_code_update();
-
-					len = ctx->on_buffer_ready(stream, ctx);
-					/*
-					 * We don't check the return value here since if we get
-					 * a negative len, it means an error occured thus we
-					 * simply remove it from the poll set and free the
-					 * stream.
-					 */
-				} while (len > 0);
-
-				/* It's ok to have an unavailable sub-buffer */
-				if (len < 0 && len != -EAGAIN && len != -ENODATA) {
-					/* Clean up stream from consumer and free it. */
-					lttng_poll_del(&events, stream->wait_fd);
-					consumer_del_metadata_stream(stream, metadata_ht);
-				}
+			} else {
+				ERR("Unexpected poll events %u for sock %d", revents, pollfd);
+				rcu_read_unlock();
+				goto end;
 			}
-
 			/* Release RCU lock for the stream looked up */
 			rcu_read_unlock();
 		}
@@ -2788,21 +2823,16 @@ restart:
 			}
 
 			if (pollfd == ctx->consumer_channel_pipe[0]) {
-				if (revents & (LPOLLERR | LPOLLHUP)) {
-					DBG("Channel thread pipe hung up");
-					/*
-					 * Remove the pipe from the poll set and continue the loop
-					 * since their might be data to consume.
-					 */
-					lttng_poll_del(&events, ctx->consumer_channel_pipe[0]);
-					continue;
-				} else if (revents & LPOLLIN) {
+				if (revents & LPOLLIN) {
 					enum consumer_channel_action action;
 					uint64_t key;
 
 					ret = read_channel_pipe(ctx, &chan, &key, &action);
 					if (ret <= 0) {
-						ERR("Error reading channel pipe");
+						if (ret < 0) {
+							ERR("Error reading channel pipe");
+						}
+						lttng_poll_del(&events, ctx->consumer_channel_pipe[0]);
 						continue;
 					}
 
@@ -2819,7 +2849,7 @@ restart:
 						rcu_read_unlock();
 						/* Add channel to the global poll events list */
 						lttng_poll_add(&events, chan->wait_fd,
-								LPOLLIN | LPOLLPRI);
+								LPOLLERR | LPOLLHUP);
 						break;
 					case CONSUMER_CHANNEL_DEL:
 					{
@@ -2879,6 +2909,17 @@ restart:
 						ERR("Unknown action");
 						break;
 					}
+				} else if (revents & (LPOLLERR | LPOLLHUP)) {
+					DBG("Channel thread pipe hung up");
+					/*
+					 * Remove the pipe from the poll set and continue the loop
+					 * since their might be data to consume.
+					 */
+					lttng_poll_del(&events, ctx->consumer_channel_pipe[0]);
+					continue;
+				} else {
+					ERR("Unexpected poll events %u for sock %d", revents, pollfd);
+					goto end;
 				}
 
 				/* Handle other stream */
@@ -2917,6 +2958,10 @@ restart:
 						&& !uatomic_read(&chan->nb_init_stream_left)) {
 					consumer_del_channel(chan);
 				}
+			} else {
+				ERR("Unexpected poll events %u for sock %d", revents, pollfd);
+				rcu_read_unlock();
+				goto end;
 			}
 
 			/* Release RCU lock for the channel looked up */
